@@ -2,7 +2,7 @@
  *
  * Information about ssh channels.
  *
- * $Id: channel.h,v 1.47 1999/12/07 00:26:19 nisse Exp $
+ * $Id: channel.h,v 1.48 2000/01/12 22:48:16 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -247,13 +247,14 @@ struct exception *make_channel_open_exception(UINT32 error_code, char *msg);
      (vars
        (handler method void
                 "struct ssh_connection *connection"
+		"UINT32 type"
                 "struct simple_buffer *data"
                 "struct command_continuation *c"
 		"struct exception_handler *e")))
 */
 
-#define CHANNEL_OPEN(o, c, d, r, e) \
-((o)->handler((o), (c), (d), (r), (e)))
+#define CHANNEL_OPEN(o, c, t, d, r, e) \
+((o)->handler((o), (c), (t), (d), (r), (e)))
 
 /* SSH_MSG_CHANNEL_REQUEST */
 /* GABA:
